local AddOn, NS = ...
local strfind, strmatch = string.find, string.match
local mathfloor, mathceil = math.floor, math.ceil
local limitDurationUpdates
function DUF_InitializeDurationUpdates()
	limitDurationUpdates = DUF_Settings[DUF_INDEX].limitdurationupdates
end

local function DUF_Aura_timeLeft(self, framename, timeLeft, settings)
	if (not DUF_INITIALIZED) then return end
	local frame = _G[framename]
	if limitDurationUpdates then 
		local timeleft = mathfloor(timeLeft)
		if (timeleft == 0) then
			frame.timeleft = 0
		else
			if (not frame.timeleft or frame.timeleft < timeleft) then
				frame.timeleft = timeleft
			end
		end
		if (frame.timeleft and timeleft and frame.timeleft - timeleft < 1) then return end
		frame.timeleft = timeleft
	end		
	if (frame.flashing) then
		if (frame.timer < 0) then
			frame.timer = .5
			if (frame.direction) then
				frame.direction = nil
			else
				frame.direction = true
			end
		end
		if (frame.direction) then
			frame:SetAlpha(frame.timer + .5)
		else
			frame:SetAlpha(1 - frame.timer)
		end
	end
	local buffText = frame.Duration
 	if (timeLeft and timeLeft > 0) then
		timeLeft = mathfloor(timeLeft)
		if (settings.altformat) then
			if (timeLeft > 60) then
				local hours = ""
				if (timeLeft >= 3600) then
					local numHours = mathfloor(timeLeft / 3600)
					hours = tostring(numHours) .. ":"
					timeLeft = timeLeft - (numHours * 3600)
				end
				local minutes = mathfloor(timeLeft / 60)
				local seconds = timeLeft - mathfloor(timeLeft / 60) * 60
				local minspacer = ""
				local secspacer = ":"
				if (minutes < 10) then
					minspacer = "0"
				end
				if (seconds < 10) then
					secspacer = ":0"
				end
				buffText:SetText(hours .. minspacer .. minutes .. secspacer .. seconds)
			else
				if (timeLeft < 10) then
					buffText:SetText("0:0" .. timeLeft .. " s")
				else
					buffText:SetText("0:" .. timeLeft .. " s")
				end
			end
		else
			buffText:SetFormattedText(SecondsToTimeAbbrev(timeLeft))
		end			
		if ((not untilCancelled) and timeLeft < 61 and timeLeft > 0) then
			if (not self.flashing) then
				frame.flashing = true
				frame.timer = .5
				frame.direction = 1
			end
		elseif (frame.flashing) then
			if (not settings.flash) then
				frame.flashing = nil
				frame:SetAlpha(1)
			end
		end
	elseif (DUF_OPTIONS_VISIBLE) then
		if (settings.altformat) then
			buffText:SetText("99:99")
		else
			buffText:SetText("99 m")
		end
	else
		buffText:SetText("")
	end
	
end

local function DUF_AnimatePortrait(self, elapsed, sequence, alpha)
	local frame = _G[DUF_FRAME_DATA[self.unit].frame .. "_Portrait"]
	frame.Model.seqtime = frame.Model.seqtime + (elapsed * 1000)
	frame:SetAlpha(alpha)
	frame.Model:SetSequenceTime(sequence, frame.Model.seqtime)
end

local function DUF_UnitFrame_HookTempEnchants(te_index, duf_index, settings)
	local button = _G["TempEnchant" .. te_index]
	if not button then return end
	local icon, count, duration, border = button:GetRegions()
	local frame = _G[DUF_FRAME_DATA["player"].frame.."_TempEnchants"]
	local duf_button = _G[frame:GetName().."_" .. duf_index]
	button:ClearAllPoints()
	button:SetPoint("TOPLEFT", duf_button, "TOPLEFT", 0, 0)
	button:SetWidth(settings.size)
	button:SetHeight(settings.size)
	border:SetWidth(settings.size)
	border:SetHeight(settings.size)
	duration:SetAlpha(0)
	local duf_duration = _G[frame:GetName().."_"..duf_index].Duration
	duf_duration:SetFont(NS:GetResourcePath(settings.durationfont, true), settings.durationheight)
	duf_duration:SetTextColor(settings.durationcolor.r, settings.durationcolor.g, settings.durationcolor.b, settings.durationalpha)
	duf_duration:ClearAllPoints()
	duf_duration:SetPoint("TOP", button, "BOTTOM", settings.durationx, settings.durationy)
	if (settings.showduration) then
		duf_duration:Show()
	else
		duf_duration:Hide()
	end
	duf_button._TEHooked = true
end

local function DUF_UnitFrame_UpdateTempEnchants(self)
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA["player"].index].TempEnchants
	local tempenchantcount = 0
	local tempenchant, frame, expirationTime, handEnchanted, timeLeft
--	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges, hasThrownEnchant, thrownExpiration, thrownCharges = GetWeaponEnchantInfo()
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, mainHandEnchantID, hasOffHandEnchant, offHandExpiration, offHandCharges, offHandEnchantId, hasRangedEnchant, rangedExpiration, rangedCharges, rangedEnchantId = GetWeaponEnchantInfo()
	local enchantIndex = 0
	local i
	for  i=3, 1, -1 do -- Fizzlemizz: Coment: Blizz allocated TE buffs in reverse order Ranged then Offhand then Main
		frame = _G[DUF_FRAME_DATA["player"].frame.."_TempEnchants_" .. i]
		if (i == 1) then
			handEnchanted = hasMainHandEnchant
			expirationTime = mainHandExpiration
		elseif (i == 2) then
			handEnchanted = hasOffHandEnchant
			expirationTime = offHandExpiration
		else
			handEnchanted = rangedExpiration
			expirationTime = rangedExpiration
		end
	
		if not expirationTime then
			timeLeft = 0
		else
			timeLeft = expirationTime / 1000 
		end
		if handEnchanted then
			enchantIndex = enchantIndex + 1-- Fizzlemizz: align the dynamic Blizz TE frame to the fixed DUF TE frame
			if not frame._TEHooked then
				DUF_UnitFrame_HookTempEnchants(enchantIndex, i, settings)
			end
			if (not frame:IsVisible()) then
				frame:Show()
			end
			tempenchantcount = i
			DUF_Aura_timeLeft(self, DUF_FRAME_DATA["player"].frame.."_TempEnchants_"..i, timeLeft, settings)
		else
			if (not DUF_OPTIONS_VISIBLE and frame:IsVisible()) then
				frame:Hide()
			end
			tempenchantcount = i
		end
	end
	frame = _G[DUF_FRAME_DATA["player"].frame.."_TempEnchants"]
	if (tempenchantcount == 0) then
		frame.tempenchantcount = nil
	elseif (not settings.hide) then
		frame.tempenchantcount = tempenchantcount
	end
end

local function DUF_UnitFrame_UpdateBuffs(self, unit)
	local buffcount = 0
	local timeLeft
	local BUFF_SETTINGS = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].Buffs
	local buff, button, icon, count
	for j=1,DL_UNIT_BUFFCOUNT do
		local i = j
		if (BUFF_SETTINGS.reverse) then
			i = DL_UNIT_BUFFCOUNT + 1 - j
		end
		button = _G[DUF_FRAME_DATA[unit].frame.."_Buffs_"..i]
		if (j <= BUFF_SETTINGS.shown) then
-- BFA			local name, rank, buff, count, buffType, duration, expirationTime, isMine, isStealable = UnitBuff(unit, j)
			local name, buff, count, buffType, duration, expirationTime, isMine, isStealable, _, spellId = UnitBuff(unit, j)
			if not expirationTime then
				timeLeft = 0
			else
				timeLeft = expirationTime - GetTime()
			end

			icon = _G[DUF_FRAME_DATA[unit].frame.."_Buffs_"..i.."_Texture"]
			if (buff) then
				icon:SetTexture(buff)
				button:Show()
--				button:SetAlpha(0)
				buffcount = j
				DUF_Aura_timeLeft(self, DUF_FRAME_DATA[unit].frame.."_Buffs_"..i, timeLeft, BUFF_SETTINGS)
				local stack = _G[DUF_FRAME_DATA[unit].frame.."_Buffs_"..i].Stack
				if (count and count > 1) then 
					stack:Show()
					stack:SetText(count)
				else
					stack:Hide()
				end
			elseif (DUF_OPTIONS_VISIBLE) then
				icon:SetTexture("Interface/AddOns/DiscordUnitFrames/Icons/Buff")
 				button:Show()
-- 				button:SetAlpha(1)
				buffcount = j
			else
				button:Hide()
--				button:SetAlpha(0)
			end
		else
			button:Hide()
--			button:SetAlpha(0)
		end
	end
	local background = _G[DUF_FRAME_DATA[unit].frame.."_Buffs"]
	if (buffcount == 0) then
--		background:Hide()
		background:SetAlpha(0)
		background:SetHeight(.5)
		background:SetWidth(.5)
		background.buffcount = 0
		return
	elseif (not BUFF_SETTINGS.hide) then
		background.buffcount = buffcount
--		background:Show()
		background:SetAlpha(BUFF_SETTINGS.alpha)
	else
		return
	end
	local size = BUFF_SETTINGS.size
	local hspacing = BUFF_SETTINGS.hspacing
	local vspacing = BUFF_SETTINGS.vspacing
	local height, width
	if (BUFF_SETTINGS.tworows) then
		local row2start = mathceil(BUFF_SETTINGS.shown / 2)
		local rows = 1
		if (buffcount > row2start) then
			buffcount = row2start
			rows = 2
		end
		if (BUFF_SETTINGS.vertical) then
			height = size * buffcount + vspacing * (buffcount - 1)
			width = size * rows + hspacing * (rows - 1)
		else
			width = size * buffcount + hspacing * (buffcount - 1)
			height = size * rows + vspacing * (rows - 1)
		end
	else
		if (BUFF_SETTINGS.vertical) then
			height = size * buffcount + vspacing * (buffcount - 1)
			width = size
		else
			width = size * buffcount + hspacing * (buffcount - 1)
			height = size
		end
	end
	background:SetWidth(width)
	background:SetHeight(height)
	background = _G[DUF_FRAME_DATA[unit].frame.."_Buffs"].Background
	background:SetWidth(width + BUFF_SETTINGS.bgpadding * 2)
	background:SetHeight(height + BUFF_SETTINGS.bgpadding * 2)
end

local function DUF_UnitFrame_UpdateDebuffs(self, unit)
	local DEBUFF_SETTINGS = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].Debuffs
	local buffcount = 0
	local buff, button, icon, debuffApplications, color, debuffType, border, timeLeft
	for j=1,DL_UNIT_BUFFCOUNT do
		local i = j
		if (DEBUFF_SETTINGS.reverse) then
			i = DL_UNIT_BUFFCOUNT + 1 - j
		end
		button = _G[DUF_FRAME_DATA[unit].frame.."_Debuffs_"..i]
		border = _G[DUF_FRAME_DATA[unit].frame.."_Debuffs_"..i.."Border"]
		if (j <= DEBUFF_SETTINGS.shown) then
-- BFA			local name, rank, buff, count, debuffType, duration, expirationTime, isMine, isStealable = UnitDebuff(unit, j)
			local name, buff, count, debuffType, duration, expirationTime, isMine, isStealable, _, spellId = UnitDebuff(unit, j)
			if not expirationTime then
				timeLeft = 0
			else
				timeLeft = expirationTime - GetTime()
			end
			if ( debuffType ) then
				color = DebuffTypeColor[debuffType]
			else
				color = DebuffTypeColor["none"]
			end
			border:SetVertexColor(color.r, color.g, color.b)
			icon = _G[DUF_FRAME_DATA[unit].frame.."_Debuffs_"..i.."_Texture"]
			if (buff) then
				icon:SetTexture(buff)
				button:Show()
--				button:SetAlpha(1)
				buffcount = j
				DUF_Aura_timeLeft(self, DUF_FRAME_DATA[unit].frame.."_Debuffs_"..i, timeLeft, DEBUFF_SETTINGS)
			elseif (DUF_OPTIONS_VISIBLE) then
				icon:SetTexture("Interface\\AddOns\\DiscordUnitFrames\\Icons\\Debuff")
				button:Show()
--				button:SetAlpha(1)
				buffcount = j
				count = 5		
			else
				button:Hide()
--				button:SetAlpha(0)
			end
			local stack = _G[DUF_FRAME_DATA[unit].frame.."_Debuffs_"..i].Stack
			if (count ~= nil) and (count > 1) then 
				stack:SetText(count)
			else
				stack:SetText("")
			end
		else
			button:Hide()
--			button:SetAlpha(0)
		end
	end
	local background = _G[DUF_FRAME_DATA[unit].frame.."_Debuffs"]
	if (buffcount == 0) then
--		background:Hide()
		background:SetAlpha(0)
		background:SetHeight(.5)
		background:SetWidth(.5)
		background.buffcount = 0
		return
	elseif (not DEBUFF_SETTINGS.hide) then
		background.buffcount = buffcount
--		background:Show()
		background:SetAlpha(DEBUFF_SETTINGS.alpha)
	else
		return
	end
	local size = DEBUFF_SETTINGS.size
	local hspacing = DEBUFF_SETTINGS.hspacing
	local vspacing = DEBUFF_SETTINGS.vspacing
	local height, width
	if (DEBUFF_SETTINGS.tworows) then
		local row2start = mathceil(DEBUFF_SETTINGS.shown / 2)
		local rows = 1
		if (buffcount > row2start) then
			buffcount = row2start
			rows = 2
		end
		if (DEBUFF_SETTINGS.vertical) then
			height = size * buffcount + vspacing * (buffcount - 1)
			width = size * rows + hspacing * (rows - 1)
		else
			width = size * buffcount + hspacing * (buffcount - 1)
			height = size * rows + vspacing * (rows - 1)
		end
	else
		if (DEBUFF_SETTINGS.vertical) then
			height = size * buffcount + vspacing * (buffcount - 1)
			width = size
		else
			width = size * buffcount + hspacing * (buffcount - 1)
			height = size
		end
	end
	background:SetWidth(width)
	background:SetHeight(height)
	background = _G[DUF_FRAME_DATA[unit].frame.."_Debuffs"].Background
	background:SetWidth(width + DEBUFF_SETTINGS.bgpadding * 2)
	background:SetHeight(height + DEBUFF_SETTINGS.bgpadding * 2)
end

-------------------------------------------------------*** End of locals ***-----------------------------------------

function DUF_Buff_OnClick(self, button)
	if InCombatLockdown() then return end
	if (self.playerbuff and button == "RightButton") then
		local buffID = self:GetID()
		if (DUF_Settings[DUF_INDEX].player.Buffs.reverse) then
			buffID = DL_UNIT_BUFFCOUNT + 1 - buffID
		end
		CancelUnitBuff("player", buffID, "HELPFUL")
	end
end

function DUF_Buff_OnEnter(self)
	local buffID = self:GetID()
	if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self:GetParent():GetParent().unit].index].Buffs.reverse) then
		buffID = DL_UNIT_BUFFCOUNT + 1 - buffID
	end
	if (self:GetRight() > UIParent:GetRight() / 2) then
		GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	end
	if (self:GetParent():GetParent().unit=="player") then
		GameTooltip:SetUnitBuff("player", buffID, "HELPFUL")
	else
		GameTooltip:SetUnitBuff(self:GetParent():GetParent().unit, buffID)
	end
end

function DUF_Buff_OnLeave()
	GameTooltip:Hide()
end

function DUF_Debuff_OnEnter(self)
	local buffID = self:GetID()
	if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self:GetParent():GetParent().unit].index].Debuffs.reverse) then
		buffID = DL_UNIT_BUFFCOUNT + 1 - buffID
	end
	if (self:GetRight() > UIParent:GetRight() / 2) then
		GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	end
	if (self:GetParent():GetParent().unit=="player") then
		GameTooltip:SetUnitDebuff("player", buffID, "HARMFUL")
	else
		GameTooltip:SetUnitDebuff(self:GetParent():GetParent().unit, buffID)
	end
end

function DUF_Debuff_OnLeave()
	GameTooltip:Hide()
end

function DUF_UnitFrame_OnDragStart(self)
	if (DUF_FRAMES_UNLOCKED) then
		self:StartMoving()
		self.moving = true
	end
end

function DUF_UnitFrame_OnDragStop(self)
	if (DUF_FRAMES_UNLOCKED) then
		self:StopMovingOrSizing()
		self.moving = nil
		local frameindex = DUF_FRAME_DATA[self.unit].index
--		local xoffset, yoffset = DL_Get_Offsets(self, UIParent, "TOPLEFT", "TOPLEFT")
		local point, relative, topoint, xoffset, yoffset = self:GetPoint(1)
--		DUF_Settings[DUF_INDEX][frameindex].xos[self:GetID()] = xoffset
--		DUF_Settings[DUF_INDEX][frameindex].yos[self:GetID()] = yoffset
		DUF_Settings[DUF_INDEX][frameindex].xos[self:GetID()] = DL_round(xoffset, 2)
		DUF_Settings[DUF_INDEX][frameindex].yos[self:GetID()] = DL_round(yoffset, 2)
--		self:ClearAllPoints()
--		self:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", xoffset, yoffset)
	end
end

function DUF_UnitFrame_OnEnter(self)
	local unit = self.unit
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]
	if (settings.usemouseovercolor) then
		self.Background:SetBackdropColor(settings.mouseovercolor.r, settings.mouseovercolor.g, settings.mouseovercolor.b, settings.bgalpha)
	end
	if (settings.mouseovergroup) then
		local unitFrame = self
		local unitFrameName = unitFrame:GetName()
		local element
		for index, value in pairs(DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]) do
			if (type(value) == "table") then
				if (index == "StatusBar") then
					for i = 1,6 do
						if (value[i]) then
							if (value[i].mouseovergroup == settings.mouseovergroup) then
								if (value[i].mouseover) then
--									_G[unitFrameName..DUF_Get_ElementName(index, i)]:Show()
									_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(1)
								end
								if (value[i].usemouseovercolor) then
									_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].mouseovercolor.r, value[i].mouseovercolor.g, value[i].mouseovercolor.b, value[i].bgalpha)
								end
							end
						end
					end
				elseif (index == "TextBox") then
					for i = 1,10 do
						if (value[i].mouseovergroup == settings.mouseovergroup) then
							if (value[i].mouseover) then
--								_G[unitFrameName..DUF_Get_ElementName(index, i)]:Show()
								_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(1)
							end
							if (value[i].usemouseovercolor) then
								_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].mouseovercolor.r, value[i].mouseovercolor.g, value[i].mouseovercolor.b, value[i].bgalpha)
							end
						end
					end
				else
					if (value.mouseovergroup == settings.mouseovergroup) then
						if (value.mouseover) then
--							_G[unitFrameName..DUF_Get_ElementName(index, i)]:Show()
							_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(1)
						end
						if (value.usemouseovercolor) then
							_G[unitFrameName..DUF_Get_ElementName(index)].Background:SetBackdropColor(value.mouseovercolor.r, value.mouseovercolor.g, value.mouseovercolor.b, value.bgalpha)
						end
					end
				end
			end
		end
	end
	if (settings.disabletooltip) then
		return
	end
	if (unit and UnitName(unit)) then
		if (settings.usecustomtooltip) then
			local text = settings.customtooltip
			if (text) then
				for var, value in pairs(NS.DUF_VARIABLE_FUNCTIONS) do
					if (strfind(text, var)) then
						text = value.func(text, unit, self) 
					end
				end
				GameTooltip_SetDefaultAnchor(GameTooltip, self)	
				GameTooltip:SetText(text, 1, 1, 1, 1, 1)
				GameTooltip:Show()
			end
		else
			UnitFrame_OnEnter(self)
		end
	end
end

function DUF_UnitFrame_OnHide(self)
	if (self.unit == "target") then
		PlaySound(PlaySoundKitID and "INTERFACESOUND_LOSTTARGETUNIT" or SOUNDKIT.INTERFACE_SOUND_LOST_TARGET_UNIT)
--		NS:DUF_ComboPoints_Update()
-- Legion    elseif self.unit == "player" then
-- Legion		NS:DUF_ComboPoints_Update()
	end
	if (self.moving) then
		DUF_UnitFrame_OnDragStop(self)
	end

end

function DUF_UnitFrame_OnLeave(self)
	local unit = self.unit
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]

	if (settings.usemouseovercolor) then
		self.Background:SetBackdropColor(settings.bgcolor.r, settings.bgcolor.g, settings.bgcolor.b, settings.bgalpha)
	end
	if (settings.mouseovergroup) then
		local unitFrame = self
		local unitFrameName = unitFrame:GetName()
		local element
		for index, value in pairs(DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]) do
			if (type(value) == "table") then
				if (index == "StatusBar") then
					for i = 1,6 do
						if (value[i]) then
							if (value[i].mouseovergroup == settings.mouseovergroup) then
								if (value[i].mouseover) then
--									_G[unitFrameName..DUF_Get_ElementName(index, i)]:Hide()
									_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
								end
								if (value[i].usemouseovercolor) then
									_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].bgcolor.r, value[i].bgcolor.g, value[i].bgcolor.b, value[i].bgalpha)
								end
							end
						end
					end
				elseif (index == "TextBox") then
					for i = 1,10 do
						if (value[i].mouseovergroup == settings.mouseovergroup) then
							if (value[i].mouseover) then
--								_G[unitFrameName..DUF_Get_ElementName(index, i)]:Hide()
								_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
							end
							if (value[i].usemouseovercolor) then
								_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].bgcolor.r, value[i].bgcolor.g, value[i].bgcolor.b, value[i].bgalpha)
							end
						end
					end
				else
					if (value.mouseovergroup == settings.mouseovergroup) then
						if (value.mouseover) then
--							_G[unitFrameName..DUF_Get_ElementName(index, i)]:Hide()
							_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
						end
						if (value.usemouseovercolor) then
							_G[unitFrameName..DUF_Get_ElementName(index)].Background:SetBackdropColor(value.bgcolor.r, value.bgcolor.g, value.bgcolor.b, value.bgalpha)
						end
					end
				end
			end
		end
	end
	if (UnitName(self.unit)) then
		UnitFrame_OnLeave(self)
	end
end

local EventFrame = CreateFrame("Frame", "DUF_Unit_EventFrame", UIParent)
EventFrame:SetSize(5, 5)
EventFrame.FrameData = DUF_FRAME_DATA
EventFrame:RegisterEvent("UNIT_AURA")
-- EventFrame:RegisterEvent("UNIT_COMBAT")
EventFrame:RegisterEvent("UNIT_NAME_UPDATE")
EventFrame:RegisterEvent("UNIT_PORTRAIT_UPDATE")
EventFrame:RegisterEvent("UNIT_MODEL_CHANGED")
EventFrame:RegisterEvent("UNIT_TARGET")
EventFrame:RegisterEvent("UNIT_DISPLAYPOWER")
--EventFrame:RegisterEvent("RAID_TARGET_UPDATE")
--EventFrame:RegisterEvent("UNIT_INVENTORY_CHANGED")
--EventFrame:RegisterEvent("PET_BATTLE_OPENING_START")
--EventFrame:RegisterEvent("PET_BATTLE_CLOSE")
EventFrame:RegisterEvent("UNIT_PET")
EventFrame:RegisterEvent("UNIT_PHASE")
EventFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
--EventFrame:RegisterEvent("UPDATE_FACTION")
EventFrame:RegisterEvent("GROUP_ROSTER_UPDATE")

EventFrame:RegisterEvent("UNIT_PET_EXPERIENCE")
if GetPetHappiness then  -- Might need to test another function PVP events
	EventFrame:RegisterEvent("UNIT_HAPPINESS")
	
	EventFrame:RegisterEvent("PLAYER_PVP_KILLS_CHANGED")
	EventFrame:RegisterEvent("PLAYER_PVP_RANK_CHANGED")
end
if FocusFrame then  -- FocusFrame exists
	EventFrame:RegisterEvent("UNIT_EXITED_VEHICLE")
	EventFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
end

EventFrame:SetScript("OnEvent", function(self, event, ...)
		if not DUF_INITIALIZED then return end
		local unit = ...
		if event == "PLAYER_FOCUS_CHANGED" then
			unit = "focus"
		elseif (event == "GROUP_ROSTER_UPDATE" or event == "RAID_ROSTER_UPDATE" or event == "PLAYER_ENTERING_WORLD") then
			DUF_Main_UpdatePartyMembers()
		elseif (event == "PLAYER_PVP_KILLS_CHANGED" or event == "PLAYER_PVP_RANK_CHANGED") then
			if unit and unit ~= "player" then 
				return
			end
			NS:DUF_HonorBar_Update()
		elseif (event == "UNIT_TARGET") then
			if unit == "target" then
				DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
			end
		elseif event == "UNIT_DISPLAYPOWER" then
			NS:DUF_AltManaBar_Update_Display()
		elseif event == "UNIT_PHASE" then
			NS:DUF_PhaseIcon_Update(unit)
		elseif (event == "PLAYER_TARGET_CHANGED") then
			if (DL_UnitName("target")) then
				if (not DUF_Settings[DUF_INDEX].target.hide) then
						DUF_UnitFrame_OnShow(DUF_TargetFrame)
						DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
						NS:DUF_Set_EliteTexture()
				end
				if (DL_UnitName("targettarget")) then
					if (not DUF_Settings[DUF_INDEX].targettarget.hide) then
						if (DUF_Settings[DUF_INDEX].hidetargettarget) then
							if (GetNumGroupMembers() + GetNumSubgroupMembers() > 0) then
									DUF_SetFrameUnitWatch(DUF_TargetOfTargetFrame,true)
									DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
							else
									DUF_SetFrameUnitWatch(DUF_TargetOfTargetFrame,false)
							end
						else
							DUF_UnitFrame_OnShow(DUF_TargetOfTargetFrame)
						end
					end
				end
			end
		end
		if unit == "vehicle" then
			unit = "pet"
		end
		if not self.FrameData[unit] then return end
		--if (arg1 ~= self.unit) then return end
		if event == "UNIT_AURA" then
			_G[DUF_FRAME_DATA[unit].frame].updateauras = true
		elseif event == "UNIT_NAME_UPDATE" then
			NS:DUF_Set_ClassIcon(unit)
			NS:DUF_Set_RaceIcon(unit)
		elseif event == "UNIT_MODEL_CHANGED" or event == "UNIT_PORTRAIT_UPDATE" then
			NS:DUF_Set_Portrait(unit)
-- Unsure if still needed		elseif (event == "UNIT_INVENTORY_CHANGED") then
--			self.updatetempenchants = true
		elseif (event == "UNIT_HAPPINESS" or event == "UNIT_PET" or event == "UNIT_EXITED_VEHICLE") then		
			NS:DUF_HappinessIcon_Update()
		elseif (event == "UNIT_PET_EXPERIENCE") then
			NS:DUF_PetXPBar_Update()
		end
		if event == DUF_UNIT_CHANGED_EVENTS[unit] then
			if DUF_FRAME_DATA[unit] then
				DUF_UnitFrame_OnShow(_G[DUF_FRAME_DATA[unit].frame]) 
			end
		end
	end)

function DUF_UnitFrame_OnLoad(self, unit)
	self.unit = unit
	ClickCastFrames = ClickCastFrames or {}
	ClickCastFrames[self] = true
	self:SetAttribute("unit", unit)
	self:RegisterForDrag("LeftButton", "RightButton")
	self:RegisterForClicks("AnyDown")
	if self.unit == "player" then
		self:RegisterUnitEvent("UNIT_POWER_FREQUENT", "player")
	end
	
--[[ Moving to single EventFrame	
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("UNIT_COMBAT")
	self:RegisterEvent("UNIT_NAME_UPDATE")
	self:RegisterEvent("UNIT_PORTRAIT_UPDATE")
	self:RegisterEvent("UNIT_MODEL_CHANGED")
	self:RegisterEvent("RAID_TARGET_UPDATE")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("PET_BATTLE_OPENING_START")
	self:RegisterEvent("PET_BATTLE_CLOSE")
	if (DUF_UNIT_CHANGED_EVENTS[self.unit]) then
		self:RegisterEvent(DUF_UNIT_CHANGED_EVENTS[self.unit])
	end
]]--
end

function DUF_UnitFrame_OnShow(self)
	if (not DUF_INITIALIZED) then return end
	if (not DL_UnitName(self.unit)) then return end
--	if (self.PetBattleHidden) then return end
	NS:DUF_HealthBar_Update(self)
	NS:DUF_LootIcon_Update(self.unit)
	NS:DUF_LeaderIcon_Update(self.unit)
	NS:DUF_PVPIcon_Update(self.unit)
	NS:DUF_StatusIcon_Update(self.unit)
	DUF_UnitFrame_UpdateBuffs(self, self.unit)
	DUF_UnitFrame_UpdateDebuffs(self, self.unit)
	DUF_UnitFrame_UpdateTempEnchants(self)
-- Legion	NS.DUF_DemonicBar_Update(self)
	NS:DUF_Set_ClassIcon(self.unit)
	NS:DUF_Set_Portrait(self.unit)
	NS:DUF_Set_RaceIcon(self.unit)
	NS:DUF_Set_RankIcon(self.unit)
	NS:DUF_RoleIcon_Update(self.unit)
	local children = { self:GetChildren() }
	for _, child in ipairs(children) do
		DUF_Element_OnShow(child)
	end
	local background = self.Background
	if (not background) then return end
	local r, g, b
	local context = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].context
	local bgcontext = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgcontext
	local bordercontext = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bordercontext
	if (context == 1 or bgcontext == 1) then
		r, g, b = NS:DUF_Get_DifficultyColor(self.unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgcolor)
	elseif (context == 2 or bgcontext == 2) then
		r, g, b = NS:DUF_Get_ReactionColor(self.unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgcolor)
	elseif (context == 3 or bgcontext == 3) then
		r, g, b = NS:DUF_Get_ClassColor(self.unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgcolor)
	else
		r, g, b = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgcolor.r, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgcolor.g, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgcolor.b
	end
	self.basecolor = {}
	self.basecolor.r = r
	self.basecolor.g = g
	self.basecolor.b = b
	background:SetBackdropColor(r, g, b, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bgalpha)
	if (bordercontext == 1) then
		r, g, b = NS:DUF_Get_DifficultyColor(self.unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bordercolor)
	elseif (bordercontext == 2) then
		r, g, b = NS:DUF_Get_ReactionColor(self.unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bordercolor)
	elseif (bordercontext == 3) then
		r, g, b = NS:DUF_Get_ClassColor(self.unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bordercolor)
	else
		r, g, b = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bordercolor.r, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bordercolor.g, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].bordercolor.b
	end
	background:SetBackdropBorderColor(r, g, b, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index].borderalpha)
	self.mana = nil
	self.health = nil
	self.manatarget = nil
	self.healthtarget = nil
	self.demopower = nil
	if (self.unit == "target") then
		if ( UnitIsEnemy("target", "player") ) then
			PlaySound(PlaySoundKitID and "igCreatureAggroSelect" or SOUNDKIT.IG_CREATURE_AGGRO_SELECT)
		elseif ( UnitIsFriend("player", "target") ) then
			PlaySound(PlaySoundKitID and "igCharacterNPCSelect" or SOUNDKIT.IG_CHARACTER_NPC_SELECT)
		else
			PlaySound(PlaySoundKitID and "igCreatureNeutralSelect" or SOUNDKIT.IG_CREATURE_NEUTRAL_SELECT)
		end
--		NS:DUF_ComboPoints_Update(true)
	end
	local frame = DUF_FRAME_DATA[self.unit].frame
	local tb
	if (self.unit == "player") then
		tb = NS.DUF_NUM_PLAYER_TEXTBOXES
-- Legion		NS:DUF_ComboPoints_Update(true)
	else
		tb = NS.DUF_NUM_OTHERS_TEXTBOXES
	end
	for i=1,tb do
		NS:DUF_TextBox_Update(_G[frame.."_TextBox_"..i])
	end
end

--function DL_FocusHide(target)
--	if (not DUF_OPTIONS_VISIBLE) then
--		FocusFrame:Hide()
--	end
--end

--hooksecurefunc("FocusUnit", DL_FocusHide)


local function UpdateTargetMarkers()
	local frame = DUF_TargetFrame_Markers
	if frame.hidewhensolo then
		local inParty = GetNumGroupMembers() + GetNumSubgroupMembers()
		if inParty > 0 then
			frame:Show()
		else
			frame:Hide()
		end
	end
end

function DUF_UnitFrame_OnUpdate(self, elapsed)
	if (not DUF_INITIALIZED) then return end
	local targettarget, focus
	local FRAME_SETTINGS = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[self.unit].index]
	local FRAME_NAME = self:GetName()
--Fizzlemizz Start onupdate animation code	
	if (self:IsVisible() and self.p3d) then
		if (not self.use3Dface) then
			if (UnitIsDead(self.unit)) then
				if not self.UnitIsDead then
					self.UnitIsDead = true
					local frame = _G[DUF_FRAME_DATA[self.unit].frame .. "_Portrait"]
					frame.Model:SetAnimation(self.deathseq)
				end
---				DUF_AnimatePortrait(self, elapsed, self.deathseq, 1)
			elseif (self.unit == "player" and UnitIsAFK(self.unit)) then
				if not self.AFK then
					self.AFK = true
					local frame = _G[DUF_FRAME_DATA[self.unit].frame .. "_Portrait"]
					frame.Model:SetAnimation(96)
				end
--				DUF_AnimatePortrait(self, elapsed, 96, 1)
			elseif (UnitIsGhost(self.unit)) then	
				DUF_AnimatePortrait(self, elapsed, 5, .5)
			else
				if self.AFK or self.UnitIsDead then
					local frame = _G[DUF_FRAME_DATA[self.unit].frame .. "_Portrait"]
					frame.Model:SetAnimation(0)
				end
				self.AFK, self.UnitIsDead = nil, nil
			end
		end
	end
--Fizzlemizz End onupdate animation code	
	if (not self.timer) then 
		self.timer = DUF_Settings[DUF_INDEX].updatespeed 
		self.updatespeed = self.timer
	end
	self.timer = self.timer - elapsed
	if (self.timer < 0) then
		self.timer = self.updatespeed
		local health = UnitHealth(self.unit)
		local healthmax = UnitHealthMax(self.unit)
		local healthtarget = UnitHealth(self.unit.."target")
		local healthmaxtarget = UnitHealthMax(self.unit.."target")
		local manatarget = UnitPower(self.unit.."target")
		local manamaxtarget = UnitPowerMax(self.unit.."target")
		local targetname = UnitName(self.unit.."target")
		local mana = UnitPower(self.unit)
		local manamax = UnitPowerMax(self.unit)
		local manatype = UnitPowerType(self.unit)
		local manatypetarget = UnitPowerType(self.unit.."target")
		local status = 0
		local leader = UnitIsGroupLeader(self.unit)
		local inParty = GetNumGroupMembers() + GetNumSubgroupMembers()
		if (inParty == 0) then
			leader = nil
		end
		local rank = NS.UnitPVPRank(self.unit)
		local pvp = UnitIsPVP(self.unit)
		local pvpfree = UnitIsPVPFreeForAll(self.unit)
		local _,loot = GetLootMethod()
		local role = NS.UnitGroupRolesAssigned(self.unit)
		local level = UnitLevel(self.unit)
		local reaction = NS:DUF_Get_Reaction(self.unit)
		local optionsvisible = DUF_OPTIONS_VISIBLE
		local targetIndex
		if (self.unit == "target" or self.unit == "targettarget" or self.unit == "focus") then
			targetIndex = GetRaidTargetIndex(self.unit)
		else
			targetIndex = GetRaidTargetIndex(self.unit.."target")
		end
		local cisChanneling = false
		local cspell, cdisplayName, cicon, cstartTime, cendTime, cisTradeSkill = NS.UnitCastingInfo(self.unit)
		if (cspell == nil) then
			cisChanneling = true
			cspell, cdisplayName, cicon, cstartTime, cendTime, cisTradeSkill = NS.UnitChannelInfo(self.unit)
		end
		if ((not self.lastmana) or mana > self.lastmana) then self.lastmana = mana end
		if ((not self.lasthealth) or health > self.lasthealth) then self.lasthealth = health end
		if (targetIndex ~= self.targetIndex) then
			self.targetIndex = targetIndex
			if (targetIndex) then
				if (self.targetIcon == 1) then
					_G[FRAME_NAME.."_RankIcon"]:SetAlpha(FRAME_SETTINGS.alpha)
					SetRaidTargetIconTexture(_G[FRAME_NAME.."_RankIcon_Texture"], targetIndex)
				elseif (self.targetIcon == 2) then
					_G[FRAME_NAME.."_RaceIcon"]:SetAlpha(FRAME_SETTINGS.alpha)
					SetRaidTargetIconTexture(_G[FRAME_NAME.."_RaceIcon"].Icon, targetIndex)
				elseif (self.targetIcon == 3) then
					_G[FRAME_NAME.."_RankIcon"]:SetAlpha(FRAME_SETTINGS.alpha)
					_G[FRAME_NAME.."_RaceIcon"]:SetAlpha(FRAME_SETTINGS.alpha)
					SetRaidTargetIconTexture(_G[FRAME_NAME.."_RankIcon"].Icon, targetIndex)
					SetRaidTargetIconTexture(_G[FRAME_NAME.."_RaceIcon"].Icon, targetIndex)
				end
				UpdateTargetMarkers()
			else
				if (self.targetIcon == 1) then
					_G[FRAME_NAME.."_RankIcon"]:SetAlpha(0)
				elseif (self.targetIcon == 2) then
					_G[FRAME_NAME.."_RaceIcon"]:SetAlpha(0)
				elseif (self.targetIcon == 3) then
					_G[FRAME_NAME.."_RaceIcon"]:SetAlpha(0)
				end
			end
		end
		if ((not inParty) or inParty == 0) then
			loot = nil
		end
		if (health <= 0 or UnitIsGhost(self.unit)) then
			status = 1
		elseif (self.unit == "player" and DL_ATTACKING) then
			status = 2
		elseif (self.unit == "player" and DL_INCOMBAT) then
			status = 2
		elseif (UnitAffectingCombat(self.unit)) then
			status = 2
		elseif (self.unit == "player" and IsResting()) then
			status = 3
		end
		if (reaction ~= self.reaction) then
			self.reaction = reaction
			if (FRAME_SETTINGS.context == 2 or FRAME_SETTINGS.bgcontext == 2) then
				local r, g, b = NS:DUF_Get_ReactionColor(self.unit, self.basecolor)
				self.Background:SetBackdropColor(r, g, b, FRAME_SETTINGS.bgalpha)
			end
			if (FRAME_SETTINGS.bordercontext == 2) then
				local r, g, b = NS:DUF_Get_ReactionColor(self.unit, FRAME_SETTINGS.bordercolor)
				self.Background:SetBackdropBorderColor(r, g, b, FRAME_SETTINGS.borderalpha)
			end
		end
		if (self.health ~= health or self.healthmax ~= healthmax) then
			self.health = health
			self.healthmax = healthmax
			NS:DUF_HealthBar_Update(self)
			if (FRAME_SETTINGS.context == 4 or FRAME_SETTINGS.bgcontext == 4) then
				local r, g, b = NS:DUF_Get_HealthColor(self.unit, self.basecolor, self.health, self.healthmax)
				self.Background:SetBackdropColor(r, g, b, FRAME_SETTINGS.bgalpha)
			end
			if (FRAME_SETTINGS.bordercontext == 4) then
				local r, g, b = NS:DUF_Get_HealthColor(self.unit, FRAME_SETTINGS.bordercolor, self.health, self.healthmax)
				self.Background:SetBackdropBorderColor(r, g, b, FRAME_SETTINGS.borderalpha)
			end
		end
		if self.unit == "player" and ((NS.DUF_ISDRUID and NS.DUF_ISSHAPESHIFTED and (self.altmana ~= UnitPower(self.unit, Enum.PowerType.Mana) or self.altmanamax ~= UnitPowerMax(self.unit, Enum.PowerType.Mana))) or NS.ALTPOWERTYPES[manatype])then 
			self.altmana = UnitPower(self.unit, Enum.PowerType.Mana) 
			self.altmanamax = UnitPowerMax(self.unit, Enum.PowerType.Mana)
			NS:DUF_AltManaBar_Update(self)
		end
		if (self.mana ~= mana or self.manamax ~= manamax) then
			self.mana = mana
			self.manamax = manamax
			NS:DUF_ManaBar_Update(self)
			if (FRAME_SETTINGS.context == 5 or FRAME_SETTINGS.bgcontext == 5) then
				local r, g, b = NS:DUF_Get_ManaColor(self.unit, self.basecolor)
				self.Background:SetBackdropColor(r, g, b, FRAME_SETTINGS.bgalpha)
			end
			if (FRAME_SETTINGS.bordercontext == 5) then
				local r, g, b = NS:DUF_Get_ManaColor(self.unit, FRAME_SETTINGS.bordercolor)
				self.Background:SetBackdropBorderColor(r, g, b, FRAME_SETTINGS.borderalpha)
			end
			for i=1,10 do
				if (FRAME_SETTINGS.TextBox[i].hideifnomana) then
					if ((not self.mana) or (not self.manamax) or self.manamax == 0) then
						if (DUF_OPTIONS_VISIBLE and (not FRAME_SETTINGS.TextBox[i].hide) and (not FRAME_SETTINGS.TextBox[i].mouseover)) then
							_G[FRAME_NAME.."_TextBox_"..i]:SetAlpha(FRAME_SETTINGS.TextBox[i].alpha)
						else
							_G[FRAME_NAME.."_TextBox_"..i]:SetAlpha(0)
						end
					elseif ((not FRAME_SETTINGS.TextBox[i].hide) and (not FRAME_SETTINGS.TextBox[i].mouseover)) then
						_G[FRAME_NAME.."_TextBox_"..i]:SetAlpha(FRAME_SETTINGS.TextBox[i].alpha)
					end
				end
			end
		end
		if (self.cast ~= nil or cspell ~= nil) then
			self.cast = cspell
-- BFA			self.castrank = crank
			self.castdisplayName = cdisplayName
			self.casticon = cicon
			self.caststartTime = cstartTime
			self.castendTime = cendTime
			self.castisTradeSkill = cisTradeSkill
			self.castisChanneling = cisChanneling
			NS:DUF_CastBar_Update(self)
		end
		if (self.status ~= status or self.optionsvisible ~= optionsvisible) then
			self.status = status
			NS:DUF_StatusIcon_Update(self.unit)
		end
		if (leader ~= self.leader or self.optionsvisible ~= optionsvisible) then
			self.leader = leader
			NS:DUF_LeaderIcon_Update(self.unit)
		end
		if (self.pvp ~= pvp or self.pvpfree ~= pvpfree or self.optionsvisible ~= optionsvisible) then
			self.pvp = pvp
			self.pvpfree = pvpfree
			NS:DUF_PVPIcon_Update(self.unit)
			if (self.unit == "player") then
				if (self.pvp or self.pvpfree) then
					PlaySound(PlaySoundKitID and "igPVPUpdate" or SOUNDKIT.IG_PVP_UPDATE)
				end
			end
		end
		if (self.loot ~= loot or self.optionsvisible ~= optionsvisible) then
			self.loot = loot
			NS:DUF_LootIcon_Update(self.unit)
		end
		if role and self.role ~= role or self.optionsvisible ~= optionsvisible then
			self.role = role
			NS:DUF_RoleIcon_Update(self.unit)
			local frame = DUF_FRAME_DATA[self.unit].frame
			for i=1,10 do
				local tBox = _G[frame.."_TextBox_"..i]
				NS:DUF_TextBox_Update(tBox)
			end
		end
		if self.rank ~= rank or self.optionsvisible ~= optionsvisible then
			self.rank = rank
			NS:DUF_Set_RankIcon(self.unit)
		end
		if (self.healthtarget ~= healthtarget or self.healthmaxtarget ~= healthmaxtarget) then
			self.healthtarget = healthtarget
			self.healthmaxtarget = healthmaxtarget
			NS:DUF_TargetHealthBar_Update(self)
		end
		if (self.manatarget ~= manatarget or self.manamaxtarget ~= manamaxtarget) then
			self.manatarget = manatarget
			self.manamaxtarget = manamaxtarget
			NS:DUF_TargetManaBar_Update(self)
		end
		if (self.manatype ~= manatype) then
			self.manatype = manatype
			NS:DUF_ManaBar_UpdateType(self)
		end
		if (self.manatypetarget ~= manatypetarget) then
			self.manatypetarget = manatypetarget
			NS:DUF_TargetManaBar_UpdateType(self)
		end
		if (self.unit == "player" and (self.updatetempenchants or _G[FRAME_NAME.."_TempEnchants"].tempenchantcount)) then
			self.updatetempenchants = false
			DUF_UnitFrame_UpdateTempEnchants(self)
		end
		if ((self.updateauras) or (_G[FRAME_NAME.."_Buffs"].buffcount and _G[FRAME_NAME.."_Buffs"].buffcount > 0) or (_G[FRAME_NAME.."_Debuffs"].buffcount and _G[FRAME_NAME.."_Debuffs"].buffcount > 0)) then
			self.updateauras = nil
			DUF_UnitFrame_UpdateBuffs(self, self.unit)
			DUF_UnitFrame_UpdateDebuffs(self, self.unit)
		end
		if (self.targetname ~= targetname) then
			self.targetname = targetname
			local textbox
			for i=1,10 do
				textbox = _G[FRAME_NAME.."_TextBox_"..i]
				if (textbox.checkname) then
					if (targetname and (not FRAME_SETTINGS.TextBox[i].hide) and (not FRAME_SETTINGS.TextBox[i].mouseover)) then
						textbox:SetAlpha(FRAME_SETTINGS.TextBox[i].alpha)
					elseif (not DUF_OPTIONS_VISIBLE) then
						textbox:SetAlpha(0)
					end
				end
			end
		end
		if (self.unit == "targettarget") then
			targettarget = true
			for i=1,10 do
				NS:DUF_TextBox_Update(_G["DUF_TargetOfTargetFrame_TextBox_"..i])
			end
			DUF_UnitFrame_UpdateBuffs(DUF_TargetOfTargetFrame, "targettarget")
			DUF_UnitFrame_UpdateDebuffs(DUF_TargetOfTargetFrame, "targettarget")
		end
		if (self.unit == "focus") then
			focus = true
			for i=1,10 do
				NS:DUF_TextBox_Update(_G["DUF_FocusFrame_TextBox_"..i])
			end
			DUF_UnitFrame_UpdateBuffs(DUF_FocusFrame, "focus")
			DUF_UnitFrame_UpdateDebuffs(DUF_FocusFrame, "focus")

		end
		self.optionsvisible = optionsvisible
		if (targettarget) then
			self = DUF_TargetOfTargetFrame
		end
		if (focus) then
			self = DUF_FocusFrame
		end
		if (self.lastmanaalpha and self.lastmanaalpha > 0) then
			_G[FRAME_NAME.."_ManaBar"].Bar2:SetAlpha(self.lastmanaalpha)
			self.lastmanaalpha = self.lastmanaalpha - elapsed
		end
		if (self.lasthealthalpha and self.lasthealthalpha > 0) then
			_G[FRAME_NAME.."_HealthBar"].Bar2:SetAlpha(self.lasthealthalpha)
			self.lasthealthalpha = self.lasthealthalpha - elapsed
		end
		if (FRAME_SETTINGS.StatusBar[NS.Bar.Health].incombat or FRAME_SETTINGS.StatusBar[NS.Bar.Health].healthupdating or FRAME_SETTINGS.StatusBar[NS.Bar.Health].manaupdating) then
			local show
			if (FRAME_SETTINGS.StatusBar[NS.Bar.Health].incombat and self.status == 2) then
				show = 1
			end
			if (self.health and self.healthmax) then
				if (FRAME_SETTINGS.StatusBar[NS.Bar.Health].healthupdating and self.health < self.healthmax) then
					show = 1
				end
			end
			if (self.mana and self.manamax and FRAME_SETTINGS.StatusBar[NS.Bar.Health].manaupdating) then
				local _, class = UnitClass(self.unit)
				if (class == "WARRIOR") then
					if (self.mana > 0) then
						show = 1
					end
				else
					if (self.mana < self.manamax) then
						show = 1
					end
				end
			end
			if (show) then
				_G[FRAME_NAME.."_HealthBar"]:SetAlpha(FRAME_SETTINGS.StatusBar[NS.Bar.Health].alpha)
			else
				_G[FRAME_NAME.."_HealthBar"]:SetAlpha(0)
			end
		end
		if (FRAME_SETTINGS.StatusBar[NS.Bar.Power].incombat or FRAME_SETTINGS.StatusBar[NS.Bar.Power].healthupdating or FRAME_SETTINGS.StatusBar[NS.Bar.Power].manaupdating) then
			local show
			if (FRAME_SETTINGS.StatusBar[NS.Bar.Power].incombat and self.status == 2) then
				show = 1
			end
			if (FRAME_SETTINGS.StatusBar[NS.Bar.Power].healthupdating and self.health < self.healthmax) then
				show = 1
			end
			if (FRAME_SETTINGS.StatusBar[NS.Bar.Power].manaupdating and self.mana and self.manamax and self.mana < self.manamax) then
				show = 1
			end
			if (show) then
				_G[FRAME_NAME.."_ManaBar"]:SetAlpha(FRAME_SETTINGS.StatusBar[NS.Bar.Power].alpha)
			else
				_G[FRAME_NAME.."_ManaBar"]:SetAlpha(0)
			end
		end
		for t=1,10 do
			local settings = FRAME_SETTINGS.TextBox[t]
			if (settings.incombat or settings.healthupdating or settings.manaupdating) then
				local show
				if (settings.incombat and self.status == 2) then
					show = 1
				end
				if (settings.healthupdating and self.health and self.healthmax and self.health < self.healthmax) then
					show = 1
				end
				if (settings.manaupdating and self.mana and self.manamax and self.mana < self.manamax) then
					show = 1
				end
				if (show) then
					_G[FRAME_NAME.."_TextBox_"..t]:SetAlpha(settings.alpha)
				else
					_G[FRAME_NAME.."_TextBox_"..t]:SetAlpha(0)
				end
			end
		end
		if (FRAME_SETTINGS.flash) then
			local unithealth
			if UnitHealthMax(self.unit) == 0 then
				unithealth = UnitHealth(self.unit)
			else
				unithealth = UnitHealth(self.unit) / UnitHealthMax(self.unit)
			end
			if (unithealth <= DUF_Settings[DUF_INDEX].flashthreshold and UnitHealth(self.unit) > 0 and (not UnitIsGhost(self.unit))) then
				self.Flashing = true
				if (not self.flashTimer) then
					self.flashTimer = .5
				end
			elseif (self.Flashing) then
				self.Flashing = nil
				self.Background:SetAlpha(1)
				return
			else
				return
			end
			self.flashTimer = self.flashTimer - 10
			if (self.flashTimer < 0) then
				self.flashTimer = .5
				if (self.Direction) then
					self.Direction = nil
				else
					self.Direction = 1
				end
			else
				if (self.Direction) then
					self.Background:SetAlpha(self.flashTimer * 2)
				else
					self.Background:SetAlpha(1 - self.flashTimer * 2)
				end
			end
		end
	end 
end

function NS:DUF_UnitFrame_InitBuffs(self, unit)
	DUF_UnitFrame_UpdateBuffs(self, unit)
end

function NS:DUF_UnitFrame_InitDebuffs(self, unit)
	DUF_UnitFrame_UpdateDebuffs(self, unit)
end
